@@ -131,15 +131,24 @@ static struct netconfig_ifaddr *netconfig_ipv4_get_ifaddr(
 	case RTPROT_STATIC:
 
 		ip = l_settings_get_string(netconfig->active_settings, "IPv4",
-									"ip");
-		if (!ip)
-			return NULL;
+								"Address");
+		if (!ip) {
+			ip = l_settings_get_string(netconfig->active_settings,
+							"IPv4", "ip");
+			if (!ip)
+				return NULL;
+		}
 
 		ifaddr = l_new(struct netconfig_ifaddr, 1);
 		ifaddr->ip = ip;
 
 		netmask = l_settings_get_string(netconfig->active_settings,
-							"IPv4", "netmask");
+							"IPv4", "Netmask");
+		if (!netmask)
+			netmask = l_settings_get_string(
+						netconfig->active_settings,
+						"IPv4", "netmask");
+
 		if (netmask && inet_pton(AF_INET, netmask, &in_addr) > 0)
 			ifaddr->prefix_len = __builtin_popcountl(
 						L_BE32_TO_CPU(in_addr.s_addr));
@@ -150,7 +159,13 @@ static struct netconfig_ifaddr *netconfig_ipv4_get_ifaddr(
 
 		ifaddr->broadcast =
 			l_settings_get_string(netconfig->active_settings,
-							"IPv4", "broadcast");
+							"IPv4", "Broadcast");
+		if (!ifaddr->broadcast)
+			ifaddr->broadcast =
+				l_settings_get_string(
+						netconfig->active_settings,
+						"IPv4", "broadcast");
+
 		ifaddr->family = AF_INET;
 
 		return ifaddr;
@@ -189,12 +204,18 @@ static struct netconfig_ifaddr *netconfig_ipv4_get_ifaddr(
 static char *netconfig_ipv4_get_gateway(struct netconfig *netconfig)
 {
 	const struct l_dhcp_lease *lease;
+	char *gateway;
 
 	switch (netconfig->rtm_protocol) {
 	case RTPROT_STATIC:
+		gateway = l_settings_get_string(netconfig->active_settings,
+							"IPv4", "Gateway");
+		if (!gateway)
+			gateway = l_settings_get_string(
+						netconfig->active_settings,
+						"IPv4", "gateway");
 
-		return l_settings_get_string(netconfig->active_settings,
-							"IPv4", "gateway");
+		return gateway;
 
 	case RTPROT_DHCP:
 		lease = l_dhcp_client_get_lease(netconfig->dhcp_client);
@@ -212,12 +233,18 @@ static char **netconfig_ipv4_get_dns(struct netconfig *netconfig, uint8_t proto)
 	const struct l_dhcp_lease *lease;
 	struct in_addr in_addr;
 	char **dns_list;
-	char **p;
 
-	p = dns_list = l_settings_get_string_list(netconfig->active_settings,
-							"IPv4", "dns", ' ');
+	dns_list = l_settings_get_string_list(netconfig->active_settings,
+							"IPv4", "DNS", ' ');
+	if (!dns_list)
+		dns_list = l_settings_get_string_list(
+						netconfig->active_settings,
+						"IPv4", "dns", ' ');
+
 	if (dns_list && *dns_list) {
-		for (; *p; p++) {
+		char **p;
+
+		for (p = dns_list; *p; p++) {
 			if (inet_pton(AF_INET, *p, &in_addr) == 1)
 				continue;
 
@@ -251,6 +278,28 @@ static char **netconfig_ipv4_get_dns(struct netconfig *netconfig, uint8_t proto)
 	return NULL;
 }
 
+static char *netconfig_ipv4_get_domain_name(struct netconfig *netconfig,
+								uint8_t proto)
+{
+	const struct l_dhcp_lease *lease;
+	char *domain_name;
+
+	domain_name = l_settings_get_string(netconfig->active_settings,
+							"IPv4", "DomainName");
+	if (domain_name)
+		/* Allow to override the DHCP domain name with setting entry. */
+		return domain_name;
+
+	if (proto != RTPROT_DHCP)
+		return NULL;
+
+	lease = l_dhcp_client_get_lease(netconfig->dhcp_client);
+	if (!lease)
+		return NULL;
+
+	return l_dhcp_lease_get_domain_name(lease);
+}
+
 static struct netconfig_ifaddr *netconfig_ipv6_get_ifaddr(
 						struct netconfig *netconfig,
 						uint8_t proto)
@@ -263,9 +312,13 @@ static struct netconfig_ifaddr *netconfig_ipv6_get_ifaddr(
 	switch (proto) {
 	case RTPROT_STATIC:
 		ip = l_settings_get_string(netconfig->active_settings, "IPv6",
-									"ip");
-		if (!ip)
-			return NULL;
+								"Address");
+		if (!ip) {
+			ip = l_settings_get_string(netconfig->active_settings,
+								"IPv6", "ip");
+			if (!ip)
+				return NULL;
+		}
 
 		ifaddr = l_new(struct netconfig_ifaddr, 1);
 		ifaddr->ip = ip;
@@ -320,7 +373,11 @@ static char *netconfig_ipv6_get_gateway(struct netconfig *netconfig)
 	switch (netconfig->rtm_v6_protocol) {
 	case RTPROT_STATIC:
 		gateway = l_settings_get_string(netconfig->active_settings,
-							"IPv6", "gateway");
+							"IPv6", "Gateway");
+		if (!gateway)
+			gateway = l_settings_get_string(
+						netconfig->active_settings,
+						"IPv6", "gateway");
 
 		if (inet_pton(AF_INET6, gateway, &in6_addr) < 1) {
 			l_error("netconfig: Invalid IPv6 gateway address %s is "
@@ -347,12 +404,19 @@ static char **netconfig_ipv6_get_dns(struct netconfig *netconfig, uint8_t proto)
 {
 	struct in6_addr in6_addr;
 	char **dns_list;
-	char **p;
 
-	p = dns_list = l_settings_get_string_list(netconfig->active_settings,
-							"IPv6", "dns", ' ');
+	dns_list = l_settings_get_string_list(netconfig->active_settings,
+							"IPv6", "DNS", ' ');
+
+	if (!dns_list)
+		dns_list = l_settings_get_string_list(
+						netconfig->active_settings,
+						"IPv6", "dns", ' ');
+
 	if (dns_list && *dns_list) {
-		for (; *p; p++) {
+		char **p;
+
+		for (p = dns_list; *p; p++) {
 			if (inet_pton(AF_INET6, *p, &in6_addr) == 1)
 				continue;
 
@@ -685,6 +749,7 @@ static void netconfig_ipv4_ifaddr_add_cmd_cb(int error, uint16_t type,
 	struct netconfig *netconfig = user_data;
 	struct netconfig_ifaddr *ifaddr;
 	char **dns;
+	char *domain_name;
 
 	if (error && error != -EEXIST) {
 		l_error("netconfig: Failed to add IP address. "
@@ -709,12 +774,21 @@ static void netconfig_ipv4_ifaddr_add_cmd_cb(int error, uint16_t type,
 	dns = netconfig_ipv4_get_dns(netconfig, netconfig->rtm_protocol);
 	if (!dns) {
 		l_error("netconfig: Failed to obtain DNS addresses.");
-		goto done;
+		goto domain_name;
 	}
 
 	resolve_add_dns(netconfig->ifindex, ifaddr->family, dns);
 	l_strv_free(dns);
 
+domain_name:
+	domain_name = netconfig_ipv4_get_domain_name(netconfig,
+						netconfig->rtm_protocol);
+	if (!domain_name)
+		goto done;
+
+	resolve_add_domain_name(netconfig->ifindex, domain_name);
+	l_free(domain_name);
+
 done:
 	netconfig_ifaddr_destroy(ifaddr);
 }
